@@ -2,8 +2,10 @@ import abc
 import asyncio
 import functools
 import json
+import os
+import stat
 from enum import Enum
-from io import FileIO
+from mimetypes import guess_type
 from random import choices as random_choices
 from typing import (
     Any,
@@ -15,23 +17,24 @@ from typing import (
     Generic,
     Iterable,
     Iterator,
+    List,
     Mapping,
+    Optional,
     Sequence,
     Tuple,
     TypeVar,
     Union,
 )
-from typing import cast as typing_cast
-from urllib.parse import parse_qsl, quote_plus
+from urllib.parse import parse_qsl, quote
 
+from . import multipart
 from .concurrency import run_in_threadpool
-from .datastructures import URL, Address, FormData, Headers, QueryParams
+from .datastructures import URL, Address, FormData, Headers, QueryParams, UploadFile
 from .exceptions import HTTPException
-from .formparsers import AsyncMultiPartParser
 from .requests import MoreInfoFromHeaderMixin
-from .responses import BaseFileResponse, BaseResponse, build_bytes_from_sse
+from .responses import BaseResponse, FileResponseMixin, build_bytes_from_sse
 from .routing import BaseHosts, BaseRouter, BaseSubpaths
-from .typing import ASGIApp, Message, Receive, Scope, Send, ServerSentEvent
+from .typing import ASGIApp, Message, Protocol, Receive, Scope, Send, ServerSentEvent
 from .utils import cached_property
 
 
@@ -98,6 +101,15 @@ class HTTPConnection(Mapping[str, Any], MoreInfoFromHeaderMixin):
     def __len__(self) -> int:
         return len(self._scope)
 
+    def __eq__(self, other: Any) -> bool:
+        if not isinstance(other, self.__class__):
+            return NotImplemented
+        return (
+            self._scope == other._scope
+            and self._send == other._send
+            and self._receive == other._receive
+        )
+
     @cached_property
     def client(self) -> Address:
         """
@@ -225,12 +237,52 @@ class Request(HTTPConnection):
         `application/x-www-form-urlencoded`, an HTTPExcption exception will be thrown.
         """
         if self.content_type == "multipart/form-data":
-            return await AsyncMultiPartParser(self.content_type, self.stream()).parse()
+            charset = self.content_type.options.get("charset", "utf8")
+            parser = multipart.MultipartDecoder(
+                self.content_type.options["boundary"].encode("latin-1"), charset
+            )
+            field_name = ""
+            data = bytearray()
+            file: Optional[UploadFile] = None
+
+            items: List[Tuple[str, Union[str, UploadFile]]] = []
+
+            async for chunk in self.stream():
+                parser.receive_data(chunk)
+                while True:
+                    event = parser.next_event()
+                    if isinstance(event, (multipart.Epilogue, multipart.NeedData)):
+                        break
+                    elif isinstance(event, multipart.Field):
+                        field_name = event.name
+                    elif isinstance(event, multipart.File):
+                        field_name = event.name
+                        file = UploadFile(
+                            event.filename, event.headers.get("content-type", "")
+                        )
+                    elif isinstance(event, multipart.Data):
+                        if file is None:
+                            data.extend(event.data)
+                        else:
+                            await file.awrite(event.data)
+
+                        if not event.more_data:
+                            if file is None:
+                                items.append(
+                                    (field_name, multipart.safe_decode(data, charset))
+                                )
+                                data.clear()
+                            else:
+                                await file.aseek(0)
+                                items.append((field_name, file))
+                                file = None
+
+            return FormData(items)
         if self.content_type == "application/x-www-form-urlencoded":
-            data = (await self.body).decode(
+            body = (await self.body).decode(
                 encoding=self.content_type.options.get("charset", "latin-1")
             )
-            return FormData(parse_qsl(data, keep_blank_values=True))
+            return FormData(parse_qsl(body, keep_blank_values=True))
 
         raise HTTPException(
             415, {"Accpet": "multipart/form-data, application/x-www-form-urlencoded"}
@@ -359,7 +411,7 @@ class RedirectResponse(Response):
         headers: Mapping[str, str] = None,
     ) -> None:
         super().__init__(status_code=status_code, headers=headers)
-        self.headers["location"] = quote_plus(str(url), safe=":/%#?&=@[]!$&'()*+,;")
+        self.headers["location"] = quote(str(url), safe="/#%[]=:;$&()+,!?*@'~")
 
 
 class StreamResponse(Response):
@@ -382,7 +434,28 @@ class StreamResponse(Response):
         return await send_http_body(send)
 
 
-class FileResponse(BaseFileResponse, Response):
+class Sendfile(Protocol):
+    async def __call__(
+        self,
+        file_descriptor: int,
+        offset: int = None,
+        count: int = None,
+        more_body: bool = False,
+    ) -> None:
+        pass
+
+
+async def open_for_sendfile(
+    path: Union[str, bytes, "os.PathLike[str]", "os.PathLike[bytes]"]
+) -> int:
+    if os.name == "nt":
+        open_mode = os.O_RDONLY | os.O_BINARY
+    else:
+        open_mode = os.O_RDONLY
+    return await run_in_threadpool(os.open, path, open_mode)
+
+
+class FileResponse(Response, FileResponseMixin):
     """
     File response.
 
@@ -390,11 +463,97 @@ class FileResponse(BaseFileResponse, Response):
     and the range of files that need to be sent.
     """
 
-    async def handle_all(
+    def __init__(
         self,
-        send_header_only: bool,
-        file_size: int,
-        send: Send,
+        filepath: str,
+        headers: Mapping[str, str] = None,
+        content_type: str = None,
+        download_name: str = None,
+        stat_result: os.stat_result = None,
+        chunk_size: int = 4096 * 64,
+    ) -> None:
+        super().__init__(headers=headers)
+        self.filepath = filepath
+        self.content_type = (
+            content_type
+            or guess_type(download_name or os.path.basename(filepath))[0]
+            or "application/octet-stream"
+        )
+        self.download_name = download_name
+        self.stat_result = stat_result or os.stat(filepath)
+        if stat.S_ISDIR(self.stat_result.st_mode):
+            raise IsADirectoryError(f"{filepath} is a directory")
+        self.chunk_size = chunk_size
+        self.headers.update(
+            self.generate_common_headers(
+                self.filepath, self.content_type, self.download_name, self.stat_result
+            )
+        )
+
+    def create_send_or_zerocopy(self, scope: Scope, send: Send) -> Sendfile:
+        """
+        https://asgi.readthedocs.io/en/latest/extensions.html#zero-copy-send
+        """
+        if (
+            "extensions" in scope
+            and "http.response.zerocopysend" in scope["extensions"]
+        ):  # pragma: no cover
+
+            async def sendfile(
+                file_descriptor: int,
+                offset: int = None,
+                count: int = None,
+                more_body: bool = False,
+            ) -> None:
+                message = {
+                    "type": "http.response.zerocopysend",
+                    "file": file_descriptor,
+                    "more_body": more_body,
+                }
+                if offset is not None:
+                    message["offset"] = offset
+                if count is not None:
+                    message["count"] = count
+                await send(message)
+
+        else:
+
+            async def sendfile(
+                file_descriptor: int,
+                offset: int = None,
+                count: int = None,
+                more_body: bool = False,
+            ) -> None:
+                if offset is not None:
+                    await run_in_threadpool(
+                        os.lseek, file_descriptor, offset, os.SEEK_SET
+                    )
+
+                here = 0
+                should_stop = False
+                if count is None:
+                    length = self.chunk_size
+                    while not should_stop:
+                        data = await run_in_threadpool(os.read, file_descriptor, length)
+                        if len(data) == length:
+                            await send_http_body(send, data, more_body=True)
+                        else:
+                            await send_http_body(send, data, more_body=more_body)
+                            should_stop = True
+                else:
+                    while not should_stop:
+                        length = min(self.chunk_size, count - here)
+                        should_stop = length == count - here
+                        here += length
+                        data = await run_in_threadpool(os.read, file_descriptor, length)
+                        await send_http_body(
+                            send, data, more_body=more_body if should_stop else True
+                        )
+
+        return sendfile
+
+    async def handle_all(
+        self, send_header_only: bool, file_size: int, scope: Scope, send: Send
     ) -> None:
         self.headers["content-type"] = str(self.content_type)
         self.headers["content-length"] = str(file_size)
@@ -402,22 +561,18 @@ class FileResponse(BaseFileResponse, Response):
         if send_header_only:
             return await send_http_body(send)
 
-        file = typing_cast(FileIO, await run_in_threadpool(open, self.filepath, "rb"))
+        sendfile = self.create_send_or_zerocopy(scope, send)
+        file_descriptor = await open_for_sendfile(self.filepath)
         try:
-            for _ in range(0, file_size, self.chunk_size):
-                await send_http_body(
-                    send,
-                    await run_in_threadpool(file.read, self.chunk_size),
-                    more_body=True,
-                )
-            return await send_http_body(send)
+            await sendfile(file_descriptor)
         finally:
-            await run_in_threadpool(file.close)
+            await run_in_threadpool(os.close, file_descriptor)
 
     async def handle_single_range(
         self,
         send_header_only: bool,
         file_size: int,
+        scope: Scope,
         send: Send,
         start: int,
         end: int,
@@ -429,55 +584,40 @@ class FileResponse(BaseFileResponse, Response):
         if send_header_only:
             return await send_http_body(send)
 
-        file = typing_cast(FileIO, await run_in_threadpool(open, self.filepath, "rb"))
+        sendfile = self.create_send_or_zerocopy(scope, send)
+        file_descriptor = await open_for_sendfile(self.filepath)
         try:
-            await run_in_threadpool(file.seek, start)
-            for here in range(start, end, self.chunk_size):
-                await send_http_body(
-                    send,
-                    await run_in_threadpool(
-                        file.read, min(self.chunk_size, end - here)
-                    ),
-                    more_body=True,
-                )
-            return await send_http_body(send)
+            await sendfile(file_descriptor, start, end - start)
         finally:
-            await run_in_threadpool(file.close)
+            await run_in_threadpool(os.close, file_descriptor)
 
     async def handle_several_ranges(
         self,
         send_header_only: bool,
         file_size: int,
+        scope: Scope,
         send: Send,
         ranges: Sequence[Tuple[int, int]],
     ) -> None:
         boundary = "".join(random_choices("abcdefghijklmnopqrstuvwxyz0123456789", k=13))
         self.headers["content-type"] = f"multipart/byteranges; boundary={boundary}"
         content_length, generate_headers = self.generate_multipart(
-            ranges, boundary, file_size
+            ranges, boundary, file_size, self.content_type
         )
         self.headers["content-length"] = str(content_length)
         await send_http_start(send, 206, self.list_headers(as_bytes=True))
         if send_header_only:
             return await send_http_body(send)
-
-        file = typing_cast(FileIO, await run_in_threadpool(open, self.filepath, "rb"))
+        sendfile = self.create_send_or_zerocopy(scope, send)
+        file_descriptor = await open_for_sendfile(self.filepath)
         try:
             for start, end in ranges:
                 await send_http_body(send, generate_headers(start, end), more_body=True)
-                await run_in_threadpool(file.seek, start)
-                for here in range(start, end, self.chunk_size):
-                    await send_http_body(
-                        send,
-                        await run_in_threadpool(
-                            file.read, min(self.chunk_size, end - here)
-                        ),
-                        more_body=True,
-                    )
+                await sendfile(file_descriptor, start, end - start, True)
                 await send_http_body(send, b"\n", more_body=True)
             return await send_http_body(send, f"--{boundary}--\n".encode("ascii"))
         finally:
-            await run_in_threadpool(file.close)
+            await run_in_threadpool(os.close, file_descriptor)
 
     async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
         send_header_only = scope["method"] == "HEAD"
@@ -495,7 +635,7 @@ class FileResponse(BaseFileResponse, Response):
         if http_range == "" or (
             http_if_range != "" and not self.judge_if_range(http_if_range, stat_result)
         ):
-            return await self.handle_all(send_header_only, file_size, send)
+            return await self.handle_all(send_header_only, file_size, scope, send)
 
         try:
             ranges = self.parse_range(http_range, file_size)
@@ -513,11 +653,11 @@ class FileResponse(BaseFileResponse, Response):
         if len(ranges) == 1:
             start, end = ranges[0]
             return await self.handle_single_range(
-                send_header_only, file_size, send, start, end
+                send_header_only, file_size, scope, send, start, end
             )
         else:
             return await self.handle_several_ranges(
-                send_header_only, file_size, send, ranges
+                send_header_only, file_size, scope, send, ranges
             )
 
 
@@ -737,9 +877,34 @@ def request_response(view: Callable[[Request], Awaitable[Response]]) -> ASGIApp:
 
     @functools.wraps(view)
     async def asgi(scope: Scope, receive: Receive, send: Send) -> None:
-        request = Request(scope, receive, send)
-        resposne = await view(request)
-        return await resposne(scope, receive, send)
+        if scope["type"] == "websocket":
+            return await send({"type": "websocket.close", "code": 1001})
+        else:
+            request = Request(scope, receive, send)
+            resposne = await view(request)
+            return await resposne(scope, receive, send)
+
+    return asgi
+
+
+def websocket_session(view: Callable[[WebSocket], Awaitable[None]]) -> ASGIApp:
+    """
+    This can turn a callable object into a ASGI application.
+
+    ```python
+    @websocket_session
+    async def f(websocket: WebSocket) -> None:
+        ...
+    ```
+    """
+
+    @functools.wraps(view)
+    async def asgi(scope: Scope, receive: Receive, send: Send) -> None:
+        if scope["type"] == "http":
+            await Response(404)(scope, receive, send)
+        else:
+            websocket = WebSocket(scope, receive, send)
+            await view(websocket)
 
     return asgi
 
@@ -756,22 +921,20 @@ class Router(BaseRouter[ASGIApp]):
     applications = Router(
         ("/static/{filepath:any}", static_files),
         ("/api/{_:any}", api_app),
-        ("/about/{name}", about_page, "about"),
-        ("/", homepage, "homepage"),
+        ("/about/{name}", about_page),
+        ("/", homepage),
     )
     """
 
     async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
-        path = scope["path"]
-        for route in self._route_array:
-            match_up, path_params = route.matches(path)
-            if not match_up:
-                continue
+        result = self.search(scope["path"])
+        if result is None:
+            response: ASGIApp = Response(404)
+        else:
+            route, path_params = result
             scope["path_params"] = path_params
-            scope["router"] = self
-            return await route.endpoint(scope, receive, send)
-
-        return await PlainTextResponse(b"", 404)(scope, receive, send)
+            response = route.endpoint
+        return await response(scope, receive, send)
 
 
 class Subpaths(BaseSubpaths[ASGIApp]):
@@ -791,14 +954,14 @@ class Subpaths(BaseSubpaths[ASGIApp]):
 
     async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
         path = scope["path"]
-        for prefix, endpoint in self._route_array:
-            if not path.startswith(prefix):
-                continue
+        result = self.search(path)
+        if result is None:
+            response: ASGIApp = Response(404)
+        else:
+            prefix, response = result
             scope["root_path"] = scope.get("root_path", "") + prefix
             scope["path"] = path[len(prefix) :]
-            return await endpoint(scope, receive, send)
-
-        return await PlainTextResponse(b"", 404)(scope, receive, send)
+        return await response(scope, receive, send)
 
 
 class Hosts(BaseHosts[ASGIApp]):
@@ -819,9 +982,9 @@ class Hosts(BaseHosts[ASGIApp]):
         for k, v in scope["headers"]:
             if k == b"host":
                 host = v.decode("latin-1")
-        for host_pattern, endpoint in self._host_array:
-            if host_pattern.fullmatch(host) is None:
-                continue
-            return await endpoint(scope, receive, send)
-
-        return await PlainTextResponse(b"Invalid host", 404)(scope, receive, send)
+        endpoint = self.search(host)
+        if endpoint is None:
+            response: ASGIApp = PlainTextResponse(b"Invalid host", 404)
+        else:
+            response = endpoint
+        return await response(scope, receive, send)
