@@ -1,11 +1,14 @@
 import abc
 import functools
 import json
+import os
+import stat
 import time
 from concurrent.futures import FIRST_COMPLETED, ThreadPoolExecutor
-from concurrent.futures import wait as wait
+from concurrent.futures import wait as wait_futures
 from http import HTTPStatus
 from itertools import chain
+from mimetypes import guess_type
 from queue import Queue
 from random import choices as random_choices
 from typing import (
@@ -16,19 +19,29 @@ from typing import (
     Generic,
     Iterable,
     Iterator,
+    List,
     Mapping,
+    Optional,
     Sequence,
     Tuple,
     TypeVar,
     Union,
 )
-from urllib.parse import parse_qsl, quote_plus
+from urllib.parse import parse_qsl, quote
 
-from .datastructures import URL, Address, FormData, Headers, QueryParams, defaultdict
+from . import multipart
+from .datastructures import (
+    URL,
+    Address,
+    FormData,
+    Headers,
+    QueryParams,
+    UploadFile,
+    defaultdict,
+)
 from .exceptions import HTTPException
-from .formparsers import MultiPartParser
 from .requests import MoreInfoFromHeaderMixin
-from .responses import BaseFileResponse, BaseResponse, build_bytes_from_sse
+from .responses import BaseResponse, FileResponseMixin, build_bytes_from_sse
 from .routing import BaseHosts, BaseRouter, BaseSubpaths
 from .typing import Environ, Final, ServerSentEvent, StartResponse, WSGIApp
 from .utils import cached_property
@@ -60,6 +73,11 @@ class HTTPConnection(Mapping[str, Any], MoreInfoFromHeaderMixin):
     def __len__(self) -> int:
         return len(self._environ)
 
+    def __eq__(self, other: Any) -> bool:
+        if not isinstance(other, self.__class__):
+            return NotImplemented
+        return self._environ == other._environ
+
     @cached_property
     def client(self) -> Address:
         """
@@ -183,12 +201,52 @@ class Request(HTTPConnection):
         `application/x-www-form-urlencoded`, an HTTPExcption exception will be thrown.
         """
         if self.content_type == "multipart/form-data":
-            return MultiPartParser(self.content_type, self.stream()).parse()
+            charset = self.content_type.options.get("charset", "utf8")
+            parser = multipart.MultipartDecoder(
+                self.content_type.options["boundary"].encode("latin-1"), charset
+            )
+            field_name = ""
+            data = bytearray()
+            file: Optional[UploadFile] = None
+
+            items: List[Tuple[str, Union[str, UploadFile]]] = []
+
+            for chunk in self.stream():
+                parser.receive_data(chunk)
+                while True:
+                    event = parser.next_event()
+                    if isinstance(event, (multipart.Epilogue, multipart.NeedData)):
+                        break
+                    elif isinstance(event, multipart.Field):
+                        field_name = event.name
+                    elif isinstance(event, multipart.File):
+                        field_name = event.name
+                        file = UploadFile(
+                            event.filename, event.headers.get("content-type", "")
+                        )
+                    elif isinstance(event, multipart.Data):
+                        if file is None:
+                            data.extend(event.data)
+                        else:
+                            file.write(event.data)
+
+                        if not event.more_data:
+                            if file is None:
+                                items.append(
+                                    (field_name, multipart.safe_decode(data, charset))
+                                )
+                                data.clear()
+                            else:
+                                file.seek(0)
+                                items.append((field_name, file))
+                                file = None
+
+            return FormData(items)
         if self.content_type == "application/x-www-form-urlencoded":
-            data = self.body.decode(
+            body = self.body.decode(
                 encoding=self.content_type.options.get("charset", "latin-1")
             )
-            return FormData(parse_qsl(data, keep_blank_values=True))
+            return FormData(parse_qsl(body, keep_blank_values=True))
 
         raise HTTPException(
             415, {"Accpet": "multipart/form-data, application/x-www-form-urlencoded"}
@@ -313,7 +371,7 @@ class RedirectResponse(Response):
         headers: Mapping[str, str] = None,
     ) -> None:
         super().__init__(status_code=status_code, headers=headers)
-        self.headers["location"] = quote_plus(str(url), safe=":/%#?&=@[]!$&'()*+,;")
+        self.headers["location"] = quote(str(url), safe="/#%[]=:;$&()+,!?*@'~")
 
 
 class StreamResponse(Response):
@@ -339,7 +397,7 @@ class StreamResponse(Response):
             yield chunk
 
 
-class FileResponse(BaseFileResponse, Response):
+class FileResponse(Response, FileResponseMixin):
     """
     File response.
 
@@ -347,6 +405,33 @@ class FileResponse(BaseFileResponse, Response):
     and the range of files that need to be sent.
     """
 
+    def __init__(
+        self,
+        filepath: str,
+        headers: Mapping[str, str] = None,
+        content_type: str = None,
+        download_name: str = None,
+        stat_result: os.stat_result = None,
+        chunk_size: int = 4096 * 64,
+    ) -> None:
+        super().__init__(headers=headers)
+        self.filepath = filepath
+        self.content_type = (
+            content_type
+            or guess_type(download_name or os.path.basename(filepath))[0]
+            or "application/octet-stream"
+        )
+        self.download_name = download_name
+        self.stat_result = stat_result or os.stat(filepath)
+        if stat.S_ISDIR(self.stat_result.st_mode):
+            raise IsADirectoryError(f"{filepath} is a directory")
+        self.chunk_size = chunk_size
+        self.headers.update(
+            self.generate_common_headers(
+                self.filepath, self.content_type, self.download_name, self.stat_result
+            )
+        )
+
     def handle_all(
         self,
         send_header_only: bool,
@@ -396,7 +481,7 @@ class FileResponse(BaseFileResponse, Response):
         boundary = "".join(random_choices("abcdefghijklmnopqrstuvwxyz0123456789", k=13))
         self.headers["content-type"] = f"multipart/byteranges; boundary={boundary}"
         content_length, generate_headers = self.generate_multipart(
-            ranges, boundary, file_size
+            ranges, boundary, file_size, self.content_type
         )
         self.headers["content-length"] = str(content_length)
 
@@ -418,6 +503,7 @@ class FileResponse(BaseFileResponse, Response):
         self, environ: Environ, start_response: StartResponse
     ) -> Iterable[bytes]:
         send_header_only = environ["REQUEST_METHOD"] == "HEAD"
+
         stat_result = self.stat_result
         file_size = stat_result.st_size
 
@@ -456,7 +542,7 @@ class SendEventResponse(Response):
     :param ping_interval: This determines the time interval (in seconds) between sending ping messages.
     """
 
-    thread_pool = ThreadPoolExecutor(max_workers=10)
+    thread_pool = ThreadPoolExecutor(max_workers=10, thread_name_prefix="SendEvent_")
 
     required_headers = {
         "Cache-Control": "no-cache",
@@ -493,7 +579,7 @@ class SendEventResponse(Response):
         )
 
         future = self.thread_pool.submit(
-            wait,
+            wait_futures,
             (
                 self.thread_pool.submit(self.send_event),
                 self.thread_pool.submit(self.keep_alive),
@@ -551,8 +637,8 @@ class Router(BaseRouter[WSGIApp]):
     applications = Router(
         ("/static/{filepath:any}", static_files),
         ("/api/{_:any}", api_app),
-        ("/about/{name}", about_page, "about"),
-        ("/", homepage, "homepage"),
+        ("/about/{name}", about_page),
+        ("/", homepage),
     )
     ```
     """
@@ -560,16 +646,14 @@ class Router(BaseRouter[WSGIApp]):
     def __call__(
         self, environ: Environ, start_response: StartResponse
     ) -> Iterable[bytes]:
-        path = environ.get("PATH_INFO", "")
-        for route in self._route_array:
-            match_up, path_params = route.matches(path)
-            if not match_up:
-                continue
+        result = self.search(environ.get("PATH_INFO", ""))
+        if result is None:
+            response: WSGIApp = Response(404)
+        else:
+            route, path_params = result
             environ["PATH_PARAMS"] = path_params
-            environ["router"] = self
-            return route.endpoint(environ, start_response)
-
-        return PlainTextResponse(b"", 404)(environ, start_response)
+            response = route.endpoint
+        return response(environ, start_response)
 
 
 class Subpaths(BaseSubpaths[WSGIApp]):
@@ -591,14 +675,14 @@ class Subpaths(BaseSubpaths[WSGIApp]):
         self, environ: Environ, start_response: StartResponse
     ) -> Iterable[bytes]:
         path = environ.get("PATH_INFO", "")
-        for prefix, endpoint in self._route_array:
-            if not path.startswith(prefix):
-                continue
+        result = self.search(path)
+        if result is None:
+            response: WSGIApp = Response(404)
+        else:
+            prefix, response = result
             environ["SCRIPT_NAME"] = environ.get("SCRIPT_NAME", "") + prefix
             environ["PATH_INFO"] = path[len(prefix) :]
-            return endpoint(environ, start_response)
-
-        return PlainTextResponse(b"", 404)(environ, start_response)
+        return response(environ, start_response)
 
 
 class Hosts(BaseHosts[WSGIApp]):
@@ -617,10 +701,9 @@ class Hosts(BaseHosts[WSGIApp]):
     def __call__(
         self, environ: Environ, start_response: StartResponse
     ) -> Iterable[bytes]:
-        host = environ.get("HTTP_HOST", "")
-        for host_pattern, endpoint in self._host_array:
-            if host_pattern.fullmatch(host) is None:
-                continue
-            return endpoint(environ, start_response)
-
-        return PlainTextResponse(b"Invalid host", 404)(environ, start_response)
+        endpoint = self.search(environ.get("HTTP_HOST", ""))
+        if endpoint is None:
+            response: WSGIApp = PlainTextResponse(b"Invalid host", 404)
+        else:
+            response = endpoint
+        return response(environ, start_response)
